<?php

function echoai_call_service_api($endpoint, $data = [], $method = 'POST')
{

    $headers["Content-Type"] = 'application/json';
    $partner = json_decode(get_option('echoai_partner'), true);

    if ($partner) {

        $data['appId'] = $partner["appId"];
        $data['apiKey'] = $partner["apiKey"];
        $headers["Authorization: Bearer"] = $data['appId'];
        $headers["x-api-key"] = $data['apiKey'];
    }

    $api_uri_debug = $api_uri = ECHOAI_API_HOST . "/" . $endpoint;

    if ($method == "POST" || $method == "PUT") {
        $data = wp_json_encode($data);
    } else {
        $api_uri .= "?" . http_build_query($data);
    }

    $result = wp_remote_request($api_uri, [
        'method' => $method,
        'headers' => $headers,
        'body' => $data,
        'timeout' => 15,
    ]);

    if (is_wp_error($result)) {

        return ["result" => $result, "api" => $api_uri_debug];
    
    }

    return json_decode($result["body"], true);

}

add_action('save_post', function ($post_id, $post) {

    if ($post->post_status == "auto-draft" || $post->post_status == "inherit") {
        return;
    }
    $parent_id = wp_is_post_revision($post_id);
    if ($parent_id !== false) {
        $post_id = $parent_id;
    }
    return echoai_notify_service($post_id, "save", $post->post_type);

}, 10, 3);

function echoai_notify_service($post_id, $action, $type)
{

    $echoai_post_types = get_option('echoai_post_types') ?: [];
    if (!in_array($type, $echoai_post_types)) {
        return;
    }
    $post_type_object = get_post_type_object($type);

    if (!$post_type_object || $post_type_object->show_in_rest != 1) {
        return;
    }

    $rest_base = $post_type_object->rest_base ?: $post_type_object->name;

    return echoai_call_service_api("wp-webhook", [
        'postId' => strval($post_id),
        'action' => $action,
        'postType' => $rest_base,
    ]);
}

function echoai_check_api_key()
{
    $api_key = $_SERVER['HTTP_X_ECHO_API_KEY'] ?? null;
    $stored_key = get_option('echoai_wp_api_key');
    if (empty($api_key) || $api_key !== $stored_key) {
        return new WP_Error('invalid_api_key', 'Invalid API key');
    }

    return true;
}

add_action('rest_api_init', function () {

    remove_filter('rest_pre_serve_request', 'rest_send_cors_headers');
    add_filter('rest_pre_serve_request', function ($value) {
        error_log('Request Method: ' . $_SERVER['REQUEST_METHOD']);
        error_log('Origin: ' . get_http_origin());

        $origin = get_http_origin();
        if ($origin) {
            header('Access-Control-Allow-Origin: ' . esc_url_raw($origin));
            header('Access-Control-Allow-Methods: POST, GET, OPTIONS, PUT, DELETE');
            header('Access-Control-Allow-Credentials: true');
            header('Access-Control-Allow-Headers: Origin, X-Requested-With, Content-Type, Accept, Authorization, X-Echo-API-Key');
        }

        if ('OPTIONS' === $_SERVER['REQUEST_METHOD']) {
            status_header(200);
            exit();
        }

        return $value;
    });

    register_rest_route('echo-ai/v1', '/validate-hash/', [
        'methods' => ['POST', 'GET'],
        'callback' => function ($request) {

            $hash = $request->get_param('hash');
            if (empty($hash)) {
                return new WP_Error('missing_hash', 'No hash provided', ['status' => 400]);
            }
            if ($hash === get_option('echoai_hash')) {
                delete_option('echoai_hash');
                return new WP_REST_Response("ok", 200);
            } else {
                return new WP_Error('invalid_hash', 'Invalid hash', ['status' => 400]);
            }
        },
        'permission_callback' => '__return_true',
    ]);

    register_rest_route('echo-ai/v1', '/process-job/', [
        'methods' => ['POST'],
        'callback' => function ($request) {

            $echoai_check_api_key = echoai_check_api_key();
            if (is_wp_error($echoai_check_api_key)) {
                return $echoai_check_api_key;
            }

            // Call the Echo API job endpoint to process a job
            // This endpoint handles tasks such as content indexing and data updates
            // Example response payload:
            // {
            //     "max": 100,        // Total number of jobs in the queue
            //     "remaining": 50,   // Number of jobs still to be processed
            //     "planLimit": 1000, // Maximum number of jobs allowed by the current plan
            // }
            $job_result = echoai_call_service_api("job", [], "GET");
            //  return new WP_REST_Response($job_result, 200);
            if (is_wp_error($job_result)) {
                return new WP_REST_Response($job_result->errors, 401);
            }
            $index_finished = 0;

            // Check if all jobs are finished based on the remaining count
            if ($job_result["remaining"] == 0) {
                // If no jobs remaining, mark the indexing process as finished
                $index_finished = 1;
            }
            update_option('echoai_index_finished', $index_finished);
            return new WP_REST_Response($job_result, 200);
        },
        'permission_callback' => '__return_true',
    ]);

    register_rest_route('echo-ai/v1', '/setup/', [
        'methods' => ['GET', 'POST'],
        'callback' => function ($request) {
            $echoai_check_api_key = echoai_check_api_key();
            if (is_wp_error($echoai_check_api_key)) {
                return new WP_REST_Response($echoai_check_api_key->errors, 401);
            }

            if ($request->get_method() === 'GET') {
                // Return initial setup info
                $partner = json_decode(get_option('echoai_partner'), true);
                $response = [
                    "is_partner_registered" => !empty($partner),
                    "admin_email" => get_option('admin_email'),
                    "site_url" => get_bloginfo('url'),
                ];
                return new WP_REST_Response($response, 200);
            } else if ($request->get_method() === 'POST') {

                // Register partner
                if (!get_option('echoai_partner')) {

                    if (empty($request->get_param('email'))) {
                        return new WP_REST_Response(['error' => 'Email is required'], 400);
                    }

                    $hash = bin2hex(random_bytes(16));
                    update_option('echoai_hash', $hash);
                    $partner = echoai_call_service_api("partner", [
                        'email' => $request->get_param('email'),
                        'baseUrl' => get_bloginfo('url'),
                        'hash' => $hash,
                    ]);

                    if (is_wp_error($partner)) {
                        return new WP_REST_Response($partner, 500);
                    }

                    if (!empty($partner) && is_array($partner) && empty($partner["error"])) {
                        update_option('echoai_partner', wp_json_encode($partner));
                    }
                }

                // Collect and send post IDs
                $posts = echoai_collect_all_posts();

                if (is_wp_error($posts)) {
                    return new WP_REST_Response($posts->errors, 500);
                }

                $result = echoai_call_service_api("wp-webhook/batch", [
                    'posts' => $posts["posts"],
                ]);

                if (is_wp_error($result)) {
                    return new WP_REST_Response($result->errors, 500);
                }

                $response = [
                    "is_partner_registered" => true,
                    "posts_discovered" => $posts["count"],
                ];
                return new WP_REST_Response($response, 200);
            }
        },
        'permission_callback' => '__return_true',
    ]);

});
